///////////////////////////////////////////////////////////////////////////////
/*!\file    Source.c
\brief      Source zum Modul
\spezification  Dieses Source-File beinhaltet die komplette Umsetzung der 
            Funktionen des Moduls sowie modulinterne Funktionen
            und Definitionen.*/
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Senden/Beantworten eines LIN-Frame
\spezification  Die Funktion ermglicht es einen LIN-Frame zu senden bzw. zu 
            beantworten je nach angeschlossenem Prfling.
\param[in]  modulno ist die Nummer des Moduls, welches den Frame senden soll.
\param[in]  controller ist die Nummer des Controllers, der senden soll.
\param[in]  dut ist ein Pointer auf die Struktur vom Typ tDutData.
\param[in]  frame ist ein Pointer auf die Struktur vom Typ tFrame.
\param[in]  timeout ist die Zeit bis zum Auftreten eines Timeout in Sekunden.
\param[out] number ist die Anzahl empfangener und zurckgegebener Frames
\param[out] lindata ist ein Pointer auf die Struktur vom Typ tLINdata, welche 
            die empfangenen Daten der LIN-Karte enthlt.
\param[out] timeout_flag ist ein Flag, welches ein Auftreten eines Timeout 
            anzeigt.*/
///////////////////////////////////////////////////////////////////////////////
static void _SendLinFrame (int modulno, 
                           tLINTests_LinController controller, 
                           tDutData *dut, 
                           tFrame *frame,
                           double timeout,
                           int *number, 
                           tLINdata lindata[LIN_BUFFERSIZE],
                           tBoolean *timeout_flag)
{
    unsigned char data[8];  // Array zur bergabe an Funktionen fr die 
                            // LIN-Karte
    int i;                  // Zhlindex fr for-Schleife
    double delay = 500;     // Pause zwischen den Frames in ms
    double starttime;       // Startzeitpunkt der Schleife
    double time;            // letzter Zeitpunkt der Ausfhrung der Schleife
    tBoolean wait = True;   // Flag zeigt an, ob die Schleife noch weiter
                            // warten soll

    
    // Auslesen der Monitordaten der LIN-Karte
    LIN_MonitorDataRead (modulno, controller, number, lindata);
    
    // Vorbereiten der Datenbergabe
    for (i = 0; i < frame->datalengthcount; i++)
        data[i] = frame->data[i];    
    
    // falls erforderlich, Datenbytes der LIN-Karte bermitteln
    if (frame->datalengthcount > 0)
        LIN_STableSet (modulno, controller, 0, 1, &frame->identifier, 
                       &frame->datalengthcount, &data);
    
    // falls erforderlich, Sendetabelle der LIN-Karte einstellen und starten
    if (dut->type != TYPES_DUT_IS_LIN_MASTER)
    {
        LIN_MScheduleTableSet (modulno, controller, LIN_TABLE0, 
                               LIN_SET_TIME_ON, dut->baudrate, 
                               LIN_DEFAULT_SYNCHBREAK, 
                               LIN_DEFAULT_SYNCHDELIMITER,
                               LIN_DEFAULT_ARBITRATIONDELAY, 1, 
                               &frame->identifier, &delay);
        LIN_MScheduleStart (modulno, controller, LIN_TABLE0);
    }
    
    // warte bis Frame erfolgreich empfangen wurde oder ein Timeout auftritt
    starttime = Timer ();
    time = starttime;
    while ((wait) && ((time - starttime) < timeout))
    {
        time = Timer ();
        LIN_MonitorDataRead (modulno, controller, number, lindata);
        for (i = 0; i < *number; i++)
        {
            if (lindata[i].ident == frame->identifier)  
                if (((lindata[i].error == 0) && 
                    (frame->datalengthcount >= 0)) || 
                    (frame->datalengthcount < 0)) 
                    wait = False;
        }
    }
    
    // falls erforderlich Sende- und Antworttabelle lschen
    if (dut->type != TYPES_DUT_IS_LIN_MASTER)
    {
        LIN_MScheduleTableDelete (modulno, controller, LIN_TABLE0);
        if (frame->datalengthcount > 0)
            LIN_STableDelete (modulno, controller);
    }
    else
        LIN_STableDelete (modulno, controller);

    // Festellen, ob ein Timeout aufgetreten ist
    if ((starttime + timeout) <= time)
        *timeout_flag = True;
    else
        *timeout_flag = False;
}// _SendLinFrame


///////////////////////////////////////////////////////////////////////////////
/*!\brief   Senden eines CAN-Frame
\spezification  Die Funktion ermglicht es einen CAN-Frame zu senden. Es findet
            kein Auslesen des Puffers wie bei der Funktion _SendLinFrame statt,
            da es hier nicht ntig ist zu prfen, ob die passenden Datenbytes
            zum Header gesendet wurden. Werden zyklisch Nachrichten ausgegeben
            (Anzahl der Sendewiederholungen ist 0), so kann die Nachricht mit
            der Funktion _DeleteCanFrames wieder vom Bus genommen werden.
\param[in]  modulno ist die Nummer des Moduls, welches den Frame senden soll.
\param[in]  controller ist die Nummer des Controllers, der senden soll.
\param[in]  frame ist ein Pointer auf die Struktur vom Typ tFrame.
\sa         _SendLinFrame, _DeleteCanFrames*/
///////////////////////////////////////////////////////////////////////////////
static void _SendCanFrame (int modulno, 
                           tLINTests_CanController controller, 
                           tFrame *frame)
{
    unsigned char data[8];      // Array zur bergabe an Funktionen fr die 
                                // LIN-Karte
    int i;                      // Zhlindex fr for-Schleife

    
    // Vorbereiten der Datenbergabe
    for (i = 0; i < frame->datalengthcount; i++)
        data[i] = frame->data[i];
    
    // Senden des Befehls an die CAN-Karte
    CAN_DefineMessage (modulno, controller, frame->identifier, 
                       frame->cycletime, CAN_MESSAGE_SEND_AND_PREPARE,
                       frame->datalengthcount, data, frame->times_to_send);
    
    // Bei fester Anzahl an Sendewiederholungen, entsprechende Zeit warten
    SysHelp_Delay (frame->times_to_send * frame->cycletime); 
}// _SendCanFrame



///////////////////////////////////////////////////////////////////////////////
/*!\brief   Senden von LIN- oder CAN-Frames
\spezification  Diese Funktion sendet die Frames auf dem LIN- oder CAN-Bus, die
            in dem Feld der Struktur tFrame abgelegt sind. Sollte es zu einem
            Timeout beim Senden auf dem LIN-Bus kommen, so wird das Senden der
            Frames auf dem LIN-Bus abgebrochen und das zugehrige Flag gesetzt.
\param[in]  modulno_lin ist die Nummer des LIN-Moduls, welches den Frame 
            senden soll.
\param[in]  controller_lin ist die Nummer des LIN-Controllers, der senden soll.
\param[in]  dut ist ein Pointer auf die Struktur vom Typ tDutData.
\param[in]  timeout ist die Zeit bis zum Auftreten eines Timeouts beim Senden 
            eines Frames auf dem LIN-Bus, bei dem ein Response erwartet wird.
\param[in]  modulno_can ist die Nummer des CAN-Moduls, welches den Frame 
            senden soll.
\param[in]  controller_can ist die Nummer des CAN-Controllers, der senden soll.
\param[in]  number_of_frames ist die Anzahl der zu sendenden Frames.
\param[in]  frame ist ein Pointer auf eine Struktur vom Typ tFrame. Diese 
            Struktur beinhaltet die zu sendenden Frames.
\param[out] timeout_flag ist ein Flag, welches das Auftreten eines Timeouts
            anzeigt.*/
///////////////////////////////////////////////////////////////////////////////
static void _SendFrames (int modulno_lin, 
                         tLINTests_LinController controller_lin,
                         tDutData *dut,
                         double timeout,
                         int modulno_can,
                         tLINTests_CanController controller_can,
                         int number_of_frames,
                         tFrame *frame,
                         tBoolean *timeout_flag)
{
    int i;          // Zhlindex fr for-Schleife
    int number;     // Anzahl der zurckerhaltenen Daten der LIN-Karte
    tLINdata lindata[LIN_BUFFERSIZE];   // bergebene Daten von der LIN-Karte
    
    
    // for-Schleife zum Senden smtlicher Frames
    for (i = 0; i < number_of_frames; i++)
    {
        if (frame[i].type == LIN_FRAME)
        {
            if (*timeout_flag == False)
                _SendLinFrame (modulno_lin, controller_lin, dut, &frame[i], 
                               timeout, &number, lindata, timeout_flag);
        }
        else
            _SendCanFrame (modulno_can, controller_can, &frame[i]);
    }   
}// _SendFrames

